#include <LiquidCrystal.h>
#include <Wire.h>
#include "botProtocol.h"
#include "MyShiftOut.h"


//Pin connected to ST_CP of 74HC595
const int latchPin = 11;
//Pin connected to SH_CP of 74HC595
const int clockPin = 12;
////Pin connected to DS of 74HC595
int dataPin[] = {2, 3};
const int numGrps = 2;

/*
 * The LCD display circuit:
 * LCD RS pin to digital pin 10
 * LCD Enable pin to digital pin 9
 * LCD D4 pin to digital pin 8
 * LCD D5 pin to digital pin 7
 * LCD D6 pin to digital pin 6
 * LCD D7 pin to digital pin 5
 * LCD R/W pin to ground
 * 10K resistor:
 * ends to +5V and ground
 * wiper to LCD VO pin (pin 3) 
 */

LiquidCrystal lcd(10, 9, 8, 7, 6, 5);
unsigned long lastLCDUpd;

// LED display
MyShiftOut myShiftOut;

MyOutPin red1;
MyOutPin red2;
MyOutPin orange1;
MyOutPin orange2;
MyOutPin rgb_b;
MyOutPin rgb_g;
MyOutPin rgb_r;
MyOutPin uv;



const int numBars = 10;
MyOutPin bars[numBars];
MyKitt kitt;

#define indFlashRate 500
MyOutPin indLF;
MyOutPin indRF;
MyOutPin indLB;
MyOutPin indRB;


// stuff from the wire
volatile char botMode;
volatile int batLevel;
volatile int amps;
volatile int lightLF;
volatile int lightRF;
volatile int lightLB;
volatile int lightRB;
volatile int prox; 

/********************************************************************************
 */
void setup()
{
  myShiftOut.init(clockPin, latchPin, dataPin, numGrps);

  bars[0].init(0, 1, 7);
  bars[1].init(0, 1, 6);
  bars[2].init(0, 1, 5);
  bars[3].init(0, 1, 4);
  bars[4].init(0, 1, 3);
  bars[5].init(0, 1, 2);
  bars[6].init(0, 1, 1);
  bars[7].init(0, 1, 0);
  bars[8].init(0, 0, 6);
  bars[9].init(0, 0, 7);
  
  for(int i = 0; i < numBars; i++)
  {
    MyShiftOut::addPin(&bars[i]);
  }

  kitt.setPins(bars, numBars, 100);
  MyShiftOut::addFancyOut(&kitt);


  indLF.init(0, 0, 0);
  MyShiftOut::addPin(&indLF);
  indRF.init(0, 0, 1);
  MyShiftOut::addPin(&indRF);
  indLB.init(0, 0, 2);
  MyShiftOut::addPin(&indLB);
  indRB.init(0, 0, 3);
  MyShiftOut::addPin(&indRB);

  indLF.setFlash(indFlashRate);
  indRF.setSame(&indLF);
  indLB.setSame(&indLF);
  indRB.setSame(&indLF);



/*
  red1.init(0, 1, 7);
  MyShiftOut::addPin(&red1);

  red2.init(0, 1, 0);
  MyShiftOut::addPin(&red2);

  orange1.init(0, 1, 1);
  MyShiftOut::addPin(&orange1);

  orange2.init(0, 1, 2);
  MyShiftOut::addPin(&orange2);

  rgb_b.init(0, 1, 3);
  MyShiftOut::addPin(&rgb_b);

  rgb_g.init(0, 1, 4);
  MyShiftOut::addPin(&rgb_g);

  rgb_r.init(0, 1, 5);
  MyShiftOut::addPin(&rgb_r);

  uv.init(0, 1, 6);
  MyShiftOut::addPin(&uv);

    
  red1.turnOn();

  rgb_r.setCycle(&rgb_g, 1000);
  rgb_g.setCycle(&rgb_b, 500);
  rgb_b.setCycle(&rgb_r, 200);
  MyShiftOut::initCycle(&rgb_r);
  
  orange1.setFlash(750);
  orange2.setSame(&orange1);
  
  red1.setFlash(200);
  red2.setOpposite(&red1);
  
  uv.turnOn();
  
  /*
  for(int cnt = 0; cnt < numBars; cnt++)
  {
    if(cnt == 0)
    {
      bars[cnt].setCycle(&bars[cnt + 7], 70);
    }
    else
    {
      bars[cnt].setCycle(&bars[cnt - 1], 70);
    }
  }
  MyShiftOut::initCycle(&bars[0]);
  */

  // LED stuff done, now time for the i2c comms
  Wire.begin(I2C_DISPLAY);
  Wire.onReceive(receiveEvent);
  
  // LCD display stuff
  lcd.begin(20, 4);
  lcd.setCursor(0, 1);
  lcd.print("Starting ...");
  lastLCDUpd = millis();
  
  botMode = '\0';
}


/********************************************************************************
 * Remember the LCD is upside down, so the left of the screen is the right of
 * the robot and the top of the screen is the back of the robot.
 */
void loop()
{
  int volts;
  unsigned long now = millis();
  char szFmt[32];

  if(now < lastLCDUpd)
  {
    // rollover has occured
    lastLCDUpd = now;
  }
  if(now - lastLCDUpd > 200)
  {
    lcd.setCursor(0, 1);
    switch(botMode)
    {
    case CMD_REMOTE_CONTROL:
      lcd.print("Remote Control");
      break;
    case CMD_SET_PATH:
      lcd.print("Set Path      ");
      break;
    case CMD_SEEK_LIGHT:
      lcd.print("Seeking light ");
      break;
    default:
      lcd.print("Starting ...  ");
      break;
    }
    lastLCDUpd = now;
    
    // 10 would be volts if 1024
    volts = (int)((batLevel * 10) / 102.4);
    sprintf(szFmt, "%3d.%dv %4da", volts / 10, volts % 10, amps);
    lcd.setCursor(0, 2);
    lcd.print(szFmt);
    
    // display light sensors
    sprintf(szFmt, "%-4d", lightLF);
    lcd.setCursor(15, 3);
    lcd.print(szFmt);
    sprintf(szFmt, "%4d", lightRF);
    lcd.setCursor(1, 3);
    lcd.print(szFmt);
    sprintf(szFmt, "%-4d", lightLB);
    lcd.setCursor(15, 0);
    lcd.print(szFmt);
    sprintf(szFmt, "%4d", lightRB);
    lcd.setCursor(1, 0);
    lcd.print(szFmt);
    
    lcd.setCursor(19, 0);
    lcd.print((prox & LEFT_PROX_MASK) ? '*' : ' ');
    lcd.setCursor(0, 0);
    lcd.print((prox & RIGHT_PROX_MASK) ? '*' : ' ');
  }
  
  MyShiftOut::inLoop();
}


/********************************************************************************
 */
void receiveEvent(int howMany)
{
  char ch;
  int i;
  char szVal[CMD_VAL_LEN + 1];
  while(Wire.available() > 0)
  {
    ch = Wire.receive();
    switch(ch)
    {
    case CMDS_MODE:
      ch = Wire.receive();
      switch(ch)
      {
      case CMD_REMOTE_CONTROL:
      case CMD_SET_PATH:
        kitt.turnOn();
        botMode = ch;
        continue;
        
      case CMD_SEEK_LIGHT:
        indLF.turnOff();
        indLB.turnOff();
        indRF.turnOff();
        indRB.turnOff();
        kitt.turnOff();
        botMode = ch;
        continue;
      }
      break;
      
    case CMDS_POWER:
      ch = Wire.receive();
      switch(ch)
      {
      case CMD_BAT_LEVEL_RESP:
        memset(szVal, 0x00, CMD_VAL_LEN + 1);
        for(i = 0; i < CMD_VAL_LEN; i++)
        {
          szVal[i] = Wire.receive();
        }
        batLevel = atoi(szVal);
        continue;
  
      case CMD_AMP_SENS_RESP:
        memset(szVal, 0x00, CMD_VAL_LEN + 1);
        for(i = 0; i < CMD_VAL_LEN; i++)
        {
          szVal[i] = Wire.receive();
        }
        amps = atoi(szVal);
        continue;
      }
      break;

    case CMDS_OTHER_SENS:
      ch = Wire.receive();
      switch(ch)
      {
      case CMD_PROX_SENS_RESP:
        memset(szVal, 0x00, CMD_VAL_LEN + 1);
        for(i = 0; i < CMD_VAL_LEN; i++)
        {
          szVal[i] = Wire.receive();
        }
        prox = atoi(szVal);
        continue;
      }
      break;

    case CMDS_LITGHT_SENS:
      ch = Wire.receive();
      switch(ch)
      {
      case CMD_LIGHT_SENS_LF_RESP:
        memset(szVal, 0x00, CMD_VAL_LEN + 1);
        for(i = 0; i < CMD_VAL_LEN; i++)
        {
          szVal[i] = Wire.receive();
        }
        lightLF = atoi(szVal);
        continue;
  
      case CMD_LIGHT_SENS_RF_RESP:
        memset(szVal, 0x00, CMD_VAL_LEN + 1);
        for(i = 0; i < CMD_VAL_LEN; i++)
        {
          szVal[i] = Wire.receive();
        }
        lightRF = atoi(szVal);
        continue;
  
      case CMD_LIGHT_SENS_LB_RESP:
        memset(szVal, 0x00, CMD_VAL_LEN + 1);
        for(i = 0; i < CMD_VAL_LEN; i++)
        {
          szVal[i] = Wire.receive();
        }
        lightLB = atoi(szVal);
        continue;
  
      case CMD_LIGHT_SENS_RB_RESP:
        memset(szVal, 0x00, CMD_VAL_LEN + 1);
        for(i = 0; i < CMD_VAL_LEN; i++)
        {
          szVal[i] = Wire.receive();
        }
        lightRB = atoi(szVal);
        continue;
      }
      break;

    case CMDS_DRIVE:
      ch = Wire.receive();
      switch(ch)
      {
      case CMD_LEFT:
        indLF.setFlash(indFlashRate);
        indLB.setSame(&indLF);
        indRF.turnOff();
        indRB.turnOff();
        continue;
  
      case CMD_RIGHT:
        indRF.setFlash(indFlashRate);
        indRB.setSame(&indRF);
        indLF.turnOff();
        indLB.turnOff();
        continue;
  
      case CMD_LEFT_REV:
        indLB.setFlash(indFlashRate);
        indLF.turnOff();
        indRF.turnOff();
        indRB.turnOff();
        continue;
  
      case CMD_RIGHT_REV:
        indLF.turnOff();
        indLB.turnOff();
        indRF.turnOff();
        indRB.setFlash(indFlashRate);
        continue;
  
      case CMD_REVERSE:
        indLF.turnOff();
        indLB.setFlash(indFlashRate);
        indRF.turnOff();
        indRB.setSame(&indLB);
        continue;
  
      case CMD_FORWARD:
      case CMD_STOP:
        indLF.turnOff();
        indLB.turnOff();
        indRF.turnOff();
        indRB.turnOff();
        continue;
      }
      break;
    }

    // something we don't care about
    while(Wire.available() > 0)
    {
      ch = Wire.receive();
      if(ch == ETX)
      {
        break;
      }
    }
  }
}

